import setuptools

setuptools.setup(
    use_scm_version={
        "write_to": "src/opencensus_ext_newrelic/version.py",
        "write_to_template": """\
# coding: utf-8
# file generated by setuptools_scm
# don't change, don't track in version control
version = "{version!s}"
""",
    },
    install_requires=("opencensus~=0.7", "newrelic-telemetry-sdk~=0.1"),
)
